 /*
 				main.c

*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*
*	Part of:	SWarp
*
*	Author:		E.BERTIN (IAP)
*
*	Contents:	Parsing of the command line.
*
*	Last modify:	08/10/2002
*
*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/

/*
 26.08.2006:
 I included code so that input filenames can be provided in an ASCII
 file. This complements the possibility to provide those names on the
 command line. If file names are given on the command line AND in a file
 both lists are merged.
 The list is given with the new command line argument 'INPUTIMAGE_LIST'.
 It is followed by the name of an ASCII file which contains in each line
 the name of an input image.
 The modification was necessary to avoid 'Argument list too long' error 
 messages from UNIX shells. 

 07.09.2006:
 I removed unnecessary 'printf' debug information.
 */

#ifdef HAVE_CONFIG_H
#include	"config.h"
#endif

#include	<ctype.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>

#include	"define.h"
#include	"globals.h"
#include	"fits/fitscat.h"
#include	"prefs.h"

#define		SYNTAX \
"swarp image1 [image2 ...][-c <configuration_file>][-<keyword> <value>]\n" \
" or, to dump a default configuration file:\n" \
"swarp -d \n"

extern const char	notokstr[];

/********************************** main ************************************/
int	main(int argc, char *argv[])

  {
   static char	prefsname[MAXCHAR];
   char		**argkey, **argval, *str;
   char         *dummy;
   int		a, narg, nim, opt;
   FILE         *infile;

  if (argc<2)
    {
    fprintf(OUTPUT, "\n		%s  Version %s (%s)\n", BANNER,MYVERSION,DATE);
    fprintf(OUTPUT, "                (%s)\n", __PIPEVERS__);
    fprintf(OUTPUT, "\nby %s\n", COPYRIGHT);
    fprintf(OUTPUT, "visit %s\n", WEBSITE);
    error(EXIT_SUCCESS, "SYNTAX: ", SYNTAX);
    }
  QMALLOC(argkey, char *, argc);
  QMALLOC(argval, char *, argc);

/*default parameters */
  prefs.ninfield = 1;
  strcpy(prefs.infield_name[0], "image");
   strcpy(prefsname, "default.swarp");
  strcpy(prefs.inputimagelist_name, "");
  narg = nim = 0;

  for (a=1; a<argc; a++)
    {
    if (*(argv[a]) == '-')
      {
      opt = (int)argv[a][1];
      if (strlen(argv[a])<3 || opt == '-')
        {
        if (opt == '-')
          opt = (int)tolower((int)argv[a][2]);
        switch(opt)
          {
          case 'c':
            if (a<(argc-1))
              strcpy(prefsname, argv[++a]);
            break;
          case 'd':
            dumpprefs();
            exit(EXIT_SUCCESS);
            break;
          case 'v':
            fprintf(OUTPUT, "\n		%s  Version %s (%s)\n",
		BANNER,MYVERSION,DATE);
            fprintf(OUTPUT, "\nFor information, please contact: %s\n",
		COPYRIGHT);
            error(EXIT_SUCCESS, "SYNTAX: ", SYNTAX);
            break;
          case 'h':
          default:
            error(EXIT_SUCCESS,"SYNTAX: ", SYNTAX);
          }
        }
      else
        {
        argkey[narg] = &argv[a][1];
        argval[narg++] = argv[++a];
        }       
      }
    else
      {
/*---- The input image filename(s) */
      for(; (a<argc) && (*argv[a]!='-'); a++)
        for (str=NULL;str=strtok(str?NULL:argv[a], notokstr); nim++)
          if (nim<MAXINFIELD)
            strcpy(prefs.infield_name[nim], str);
          else
            error(EXIT_FAILURE, "*Error*: Too many input images: ", str);
      prefs.ninfield = nim;
      a--;
      }
    }

  readprefs(prefsname, argkey, argval, narg);

  /* test if we have a file containing input images 
     if yes, read files and add them to the image list 
  */

  if(strcmp(prefs.inputimagelist_name, "") != 0)
    {
      if ((infile = fopen(prefs.inputimagelist_name, "r")) != NULL)
	{
          /* Todo: make the file reading more robust to filenames having spaces
             at the beginning and the end of their names
	  */
	  while((dummy=fgets(prefs.infield_name[nim], MAXCHAR, infile)) && (nim<MAXINFIELD))
	    {
	      /* remove possible 'newline' in filenames */
	      if((dummy=strchr(prefs.infield_name[nim], '\n')) != NULL)
		{
		  *dummy='\0';
		}
	      nim++;
	    }
	  fclose(infile);
	  if(nim == MAXINFIELD && dummy != NULL)
	    {
              error(EXIT_FAILURE, "*Error*: Too many input images: ", str);
	    }
          prefs.ninfield = nim;

	}
      else
	{
	  error(EXIT_FAILURE, "*Error*: File not present: ", prefs.inputimagelist_name);
	}
    }

  useprefs();

  free(argkey);
  free(argval);

  makeit();

  NFPRINTF(OUTPUT, "All done");
  NPRINTF(OUTPUT, "\n");

  exit(EXIT_SUCCESS);
  }

